/* ogcalc - calculate %ABV and OG from PG, RI and CF.
 *
 * Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 *
 **********************************************************/

#include "ogcalc.h"

static void
ogcalc_init ( GTypeInstance *instance,
	      gpointer       g_class );

GType
ogcalc_get_type (void)
{
  static GType type = 0;

  g_print("get_type\n");

 if (type == 0)
    {
      static const GTypeInfo info =
	{
	  sizeof (OgcalcClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) ogcalc_init,
	  NULL,
	  NULL,
	  sizeof (Ogcalc),
	  0,
	  NULL
	};
      g_print("get_type register\n");
      type = g_type_register_static (GTK_TYPE_WINDOW,
				     "Ogcalc",
				     &info,
				     0);
    }
  return type;
}

static void
ogcalc_init ( GTypeInstance *instance,
	      gpointer       g_class )
{
  Ogcalc *self = (Ogcalc *) instance;

  g_print("init\n");

  /* Set the window title */
  gtk_window_set_title(GTK_WINDOW (self), "OG & ABV Calculator");
  /* Don't permit resizing */
  gtk_window_set_resizable(GTK_WINDOW (self), FALSE);

  /* Load the interface description */
  self->xml = glade_xml_new("ogcalc.glade", NULL, NULL);

  /* Get the widgets */
  self->pg_val = GTK_SPIN_BUTTON (glade_xml_get_widget (self->xml, "pg_entry"));
  self->ri_val = GTK_SPIN_BUTTON (glade_xml_get_widget (self->xml, "ri_entry"));
  self->cf_val = GTK_SPIN_BUTTON (glade_xml_get_widget (self->xml, "cf_entry"));
  self->og_result = GTK_LABEL (glade_xml_get_widget (self->xml, "og_result"));
  self->abv_result = GTK_LABEL (glade_xml_get_widget (self->xml, "abv_result"));
  self->quit_button = GTK_BUTTON (glade_xml_get_widget (self->xml, "quit_button"));
  self->reset_button = GTK_BUTTON (glade_xml_get_widget (self->xml, "reset_button"));
  self->calculate_button = GTK_BUTTON (glade_xml_get_widget (self->xml, "calculate_button"));

  /* Set up the signal handlers */
  glade_xml_signal_autoconnect(self->xml);

  /* Get the main window and pack it into our window */
  gtk_widget_reparent (glade_xml_get_widget (self->xml, "vbox1"),
		       GTK_WIDGET (self));
  /*
   * Ensure calculate is the default.  The default was lost by
   * reparenting.
   */
  gtk_widget_grab_default(GTK_WIDGET (self->calculate_button));
}

Ogcalc *
ogcalc_new (void)
{
  return (Ogcalc *) g_object_new(OGCALC_TYPE, NULL);
}

void
ogcalc_reset ( Ogcalc *ogcalc,
	       gpointer   data )
{
  gtk_spin_button_set_value (ogcalc->pg_val, 0.0);
  gtk_spin_button_set_value (ogcalc->ri_val, 0.0);
  gtk_spin_button_set_value (ogcalc->cf_val, 0.0);
  gtk_label_set_text (ogcalc->og_result, "");
  gtk_label_set_text (ogcalc->abv_result, "");
}

void
ogcalc_calculate ( Ogcalc *ogcalc,
		   gpointer   data )
{
  gdouble pg, ri, cf, og, abv;
  gchar *og_string;
  gchar *abv_string;

  pg = gtk_spin_button_get_value (ogcalc->pg_val);
  ri = gtk_spin_button_get_value (ogcalc->ri_val);
  cf = gtk_spin_button_get_value (ogcalc->cf_val);

  og = (ri * 2.597) - (pg * 1.644) - 34.4165 + cf;

  /* Do the sums */
  if (og < 60)
    abv = (og - pg) * 0.130;
  else
    abv = (og - pg) * 0.134;

  /* Display the results.  Note the <b></b> GMarkup tags to
     make it display in Bold. */
  og_string = g_strdup_printf ("<b>%0.2f</b>", og);
  abv_string = g_strdup_printf ("<b>%0.2f</b>", abv);

  gtk_label_set_markup (ogcalc->og_result, og_string);
  gtk_label_set_markup (ogcalc->abv_result, abv_string);

  g_free (og_string);
  g_free (abv_string);
}

/*
 * Local Variables:
 * mode:C
 * fill-column:60
 * End:
 */
